/******************************************************************************
*
* Copyright (C) 2016 Xilinx, Inc.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or
* (b) that interact with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Except as contained in this notice, the name of the Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Xilinx.
*
******************************************************************************/

#include <stdio.h>
#include "xparameters.h"
#include "netif/xadapter.h"
#include "platform_config.h"
#include "xil_printf.h"

#if LWIP_IPV6==1
#include "lwip/ip.h"
#else
#if LWIP_DHCP==1
#include "lwip/dhcp.h"
#endif
#endif

#ifdef XPS_BOARD_ZCU102
#ifdef XPAR_XIICPS_0_DEVICE_ID
int IicPhyReset(void);
#endif
#endif

int main_thread();
void echo_application_thread(void *);
void xemacif_input_thread(struct netif *);
void print_echo_app_header();

void lwip_init();

#if LWIP_IPV6==0
#if LWIP_DHCP==1
extern volatile int dhcp_timoutcntr;
err_t dhcp_start(struct netif *netif);
#endif
#endif

#define THREAD_STACKSIZE 1024

static struct netif server_netif_0;
static struct netif server_netif_1;
static struct netif server_netif_2;
static struct netif server_netif_3;

SemaphoreHandle_t xSemaphore;

const char *PortA ="Port A";
const char *PortB ="Port B";
const char *PortC ="Port C";
const char *PortD ="Port D";

u16_t echo_port_A = 7;
u16_t echo_port_B = 8;
u16_t echo_port_C = 9;
u16_t echo_port_D = 10;

#if LWIP_IPV6==1
void print_ip6(char *msg, ip_addr_t *ip)
{
	print(msg);
	xil_printf(" %x:%x:%x:%x:%x:%x:%x:%x\n\r",
			IP6_ADDR_BLOCK1(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK2(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK3(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK4(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK5(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK6(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK7(&ip->u_addr.ip6),
			IP6_ADDR_BLOCK8(&ip->u_addr.ip6));
}

#else
void
print_ip(char *msg, ip_addr_t *ip)
{
	xil_printf(msg);
	xil_printf("%d.%d.%d.%d\n\r", ip4_addr1(ip), ip4_addr2(ip),
			ip4_addr3(ip), ip4_addr4(ip));
}

void
print_ip_settings(ip_addr_t *ip, ip_addr_t *mask, ip_addr_t *gw, char port)
{
	xil_printf("---------PORT %C---------\n\r", port);
	print_ip("Board IP: ", ip);
	print_ip("Netmask : ", mask);
	print_ip("Gateway : ", gw);
	xil_printf("\r\n");
}

#endif
int main()
{
	xSemaphore = xSemaphoreCreateMutex();

	sys_thread_new("main_thrd", (void(*)(void*))main_thread, 0,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);
	vTaskStartScheduler();
	while(1);
	return 0;
}

void network_thread_A(void* p)
{
	struct netif *netif_0;
	/* the mac address of the board. this should be unique per board */
	unsigned char mac_ethernet_address_0[] = { 0x00, 0x0a, 0x35, 0x00, 0x01, 0x02 };
	#if LWIP_IPV6==0
		ip_addr_t ipaddr_0, netmask_0, gw_0;
	#endif

	netif_0 = &server_netif_0;

	#if LWIP_IPV6==0
		/* initliaze IP addresses to be used */
		IP4_ADDR(&ipaddr_0,  192, 168, 5, 10);
		IP4_ADDR(&netmask_0, 255, 255, 255,  0);
		IP4_ADDR(&gw_0,      192, 168, 5, 1);
	#endif

	xSemaphoreTake(xSemaphore, 100);
	print_ip_settings(&ipaddr_0, &netmask_0, &gw_0, 'A');
	xSemaphoreGive(xSemaphore);

	#if LWIP_IPV6==0
		/* Add network interface to the netif_list, and set it as default */
		if (!xemac_add(netif_0, &ipaddr_0, &netmask_0, &gw_0, mac_ethernet_address_0, PLATFORM_EMAC_BASEADDR_0)) {
			xil_printf("Error adding N/W interface\r\n");
			return;
	    }
	#endif

	netif_set_default(netif_0);

	/* specify that the network if is up */
	netif_set_up(netif_0);

	/* start packet receive thread - required for lwIP operation */
	sys_thread_new("xemacif_input_thread", (void(*)(void*))xemacif_input_thread, netif_0,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);

	#if LWIP_IPV6==0
		xSemaphoreTake(xSemaphore, 100);
		xil_printf("\r\n");
		xil_printf("%30s %6s %s\r\n", "Server", "Port", "Connect With..");
		xil_printf("%30s %6s %s\r\n", "--------------------", "------", "--------------------");
		print_echo_app_header(PortA,echo_port_A);
		xil_printf("\r\n");
		xSemaphoreGive(xSemaphore);

		sys_thread_new("echod", echo_application_thread, echo_port_A,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);
		vTaskDelete(NULL);
	#endif
	return;
}

void network_thread_B(void* p)
{
	struct netif *netif_1;
	/* the mac address of the board. this should be unique per board */
	unsigned char mac_ethernet_address_1[] = { 0x00, 0x0a, 0x35, 0x00, 0x01, 0x03 };
	#if LWIP_IPV6==0
		ip_addr_t ipaddr_1, netmask_1, gw_1;
	#endif

	netif_1 = &server_netif_1;

	#if LWIP_IPV6==0
		/* initliaze IP addresses to be used */
		IP4_ADDR(&ipaddr_1,  192, 168, 5, 11);
		IP4_ADDR(&netmask_1, 255, 255, 255,  0);
		IP4_ADDR(&gw_1,      192, 168, 5, 1);
	#endif

	xSemaphoreTake(xSemaphore, 100);
	print_ip_settings(&ipaddr_1, &netmask_1, &gw_1, 'B');
	xSemaphoreGive(xSemaphore);

	#if LWIP_IPV6==0
		vTaskDelay( 10 );
		/* Add network interface to the netif_list, and set it as default */
		if (!xemac_add(netif_1, &ipaddr_1, &netmask_1, &gw_1, mac_ethernet_address_1, PLATFORM_EMAC_BASEADDR_1)) {
			xil_printf("Error adding N/W interface\r\n");
			return;
		}
	#endif

	netif_set_default(netif_1);

	/* specify that the network if is up */
	netif_set_up(netif_1);

	/* start packet receive thread - required for lwIP operation */
	sys_thread_new("xemacif_input_thread", (void(*)(void*))xemacif_input_thread, netif_1,
			THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);

	#if LWIP_IPV6==0
		xSemaphoreTake(xSemaphore, 100);
		xil_printf("\r\n");
		xil_printf("%30s %6s %s\r\n", "Server", "Port", "Connect With..");
		xil_printf("%30s %6s %s\r\n", "--------------------", "------", "--------------------");
		print_echo_app_header(PortB, echo_port_B);
		xil_printf("\r\n");
		xSemaphoreGive(xSemaphore);

		sys_thread_new("echod", echo_application_thread, echo_port_B,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);
		vTaskDelete(NULL);

	#endif
	return;
}

void network_thread_C(void* p)
{
	struct netif *netif_2;
	/* the mac address of the board. this should be unique per board */
	unsigned char mac_ethernet_address_2[] = { 0x00, 0x0a, 0x35, 0x00, 0x01, 0x04 };
	#if LWIP_IPV6==0
		ip_addr_t ipaddr_2, netmask_2, gw_2;
	#endif

	netif_2 = &server_netif_2;

	#if LWIP_IPV6==0
		/* initliaze IP addresses to be used */
		IP4_ADDR(&ipaddr_2,  192, 168, 5, 12);
		IP4_ADDR(&netmask_2, 255, 255, 255,  0);
		IP4_ADDR(&gw_2,      192, 168, 5, 1);
	#endif

	xSemaphoreTake(xSemaphore, 100);
	print_ip_settings(&ipaddr_2, &netmask_2, &gw_2, 'C');
	xSemaphoreGive(xSemaphore);

	#if LWIP_IPV6==0
		/* Add network interface to the netif_list, and set it as default */
		if (!xemac_add(netif_2, &ipaddr_2, &netmask_2, &gw_2, mac_ethernet_address_2, PLATFORM_EMAC_BASEADDR_2)) {
			xil_printf("Error adding N/W interface\r\n");
			return;
		}
	#endif

	netif_set_default(netif_2);

	/* specify that the network if is up */
	netif_set_up(netif_2);

	/* start packet receive thread - required for lwIP operation */
	sys_thread_new("xemacif_input_thread", (void(*)(void*))xemacif_input_thread, netif_2,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);

	#if LWIP_IPV6==0
		xSemaphoreTake(xSemaphore, 100);
		xil_printf("\r\n");
		xil_printf("%30s %6s %s\r\n", "Server", "Port", "Connect With..");
		xil_printf("%30s %6s %s\r\n", "--------------------", "------", "--------------------");
		print_echo_app_header(PortC, echo_port_C);
		xil_printf("\r\n");
		xSemaphoreGive(xSemaphore);

		sys_thread_new("echod", echo_application_thread, echo_port_C,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);
		vTaskDelete(NULL);

	#endif
	return;
}

void network_thread_D(void* p)
{
	struct netif *netif_3;
	/* the mac address of the board. this should be unique per board */
	unsigned char mac_ethernet_address_3[] = { 0x00, 0x0a, 0x35, 0x00, 0x01, 0x05 };
	#if LWIP_IPV6==0
		ip_addr_t ipaddr_3, netmask_3, gw_3;
	#endif

	netif_3 = &server_netif_3;

	#if LWIP_IPV6==0
		/* initliaze IP addresses to be used */
		IP4_ADDR(&ipaddr_3,  192, 168, 5, 13);
		IP4_ADDR(&netmask_3, 255, 255, 255,  0);
		IP4_ADDR(&gw_3,      192, 168, 5, 1);
	#endif

	xSemaphoreTake(xSemaphore, 100);
	print_ip_settings(&ipaddr_3, &netmask_3, &gw_3, 'D');
	xSemaphoreGive(xSemaphore);

	#if LWIP_IPV6==0
		/* Add network interface to the netif_list, and set it as default */
		if (!xemac_add(netif_3, &ipaddr_3, &netmask_3, &gw_3, mac_ethernet_address_3, PLATFORM_EMAC_BASEADDR_3)) {
			xil_printf("Error adding N/W interface\r\n");
			return;
		}
	#endif

	netif_set_default(netif_3);

	/* specify that the network if is up */
	netif_set_up(netif_3);

	/* start packet receive thread - required for lwIP operation */
	sys_thread_new("xemacif_input_thread", (void(*)(void*))xemacif_input_thread, netif_3,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);

	#if LWIP_IPV6==0
		xSemaphoreTake(xSemaphore, 100);
		xil_printf("\r\n");
		xil_printf("%30s %6s %s\r\n", "Server", "Port", "Connect With..");
		xil_printf("%30s %6s %s\r\n", "--------------------", "------", "--------------------");
		print_echo_app_header(PortD, echo_port_D);
		xil_printf("\r\n");
		xSemaphoreGive(xSemaphore);

		sys_thread_new("echod", echo_application_thread, echo_port_D,
			THREAD_STACKSIZE,
			DEFAULT_THREAD_PRIO);
		vTaskDelete(NULL);

	#endif
	return;
}

int main_thread()
{

#if LWIP_DHCP==1
	int mscnt = 0;
#endif

#ifdef XPS_BOARD_ZCU102
	IicPhyReset();
#endif

	/* initialize lwIP before calling sys_thread_new */
	lwip_init();

	xil_printf("\r\n\r\n");
	xil_printf("-----lwIP Socket Mode Echo server Demo Application ------\r\n\n\r");

	/* any thread using lwIP should be created using sys_thread_new */
	sys_thread_new("NW_THRD_A", network_thread_A, NULL,
		THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);

	sys_thread_new("NW_THRD_B", network_thread_B, NULL,
		THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);

	sys_thread_new("NW_THRD_C", network_thread_C, NULL,
		THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);

	sys_thread_new("NW_THRD_D", network_thread_D, NULL,
		THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);

	vTaskDelete(NULL);
	return 0;
}
